#include <extdll.h>
#include <enginecallback.h>		// ALERT()

#include "osdep.h"				// win32 vsnprintf, etc

void UTIL_LogPrintf( char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsnprintf ( string, sizeof(string), fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	ALERT( at_logged, "%s", string );
}

bool IsAlive(edict_t *pEdict)
{
   if (FNullEnt(pEdict))
      return FALSE; // reliability check

   return (pEdict->v.deadflag == DEAD_NO &&
      pEdict->v.health > 0 &&
      pEdict->v.movetype != MOVETYPE_NOCLIP);
}

// Shake the screen of all clients within radius
// radius == 0, shake all clients
void UTIL_ScreenShake( const Vector &center, float amplitude, float frequency, float duration, float radius )
{
   // This structure is sent over the net to describe a screen shake event
   struct ScreenShake
   {
      unsigned short amplitude; // FIXED 4.12 amount of shake
      unsigned short duration;  // FIXED 4.12 seconds duration
      unsigned short frequency; // FIXED 8.8 noise frequency (low frequency is a jerk,high frequency is a rumble)
   } shake;

   int i;
   float localAmplitude;

   static int gmsgShake = 0;

   if (gmsgShake == 0)
      gmsgShake = REG_USER_MSG( "ScreenShake", sizeof(ScreenShake));

   shake.duration = FixedUnsigned16( duration, 1<<12 );      // 4.12 fixed
   shake.frequency = FixedUnsigned16( frequency, 1<<8 );   // 8.8 fixed

   for ( i = 1; i <= gpGlobals->maxClients; i++ )
   {
      edict_t *pPlayer = INDEXENT(i);

      if (FNullEnt(pPlayer))
         continue;

      if (!IsAlive(pPlayer))
         continue;

      if (!(pPlayer->v.flags & FL_ONGROUND) )   // Don't shake if not onground
         continue;

      localAmplitude = 0;

      if ( radius <= 0 )
         localAmplitude = amplitude;
      else
      {
         Vector delta = center - pPlayer->v.origin;
         float distance = delta.Length();
         // Had to get rid of this falloff - it didn't work well
         if ( distance < radius )
            localAmplitude = amplitude; // radius - distance;
      }
      if ( localAmplitude )
      {
         shake.amplitude = FixedUnsigned16( localAmplitude, 1<<12 );      // 4.12 fixed
         MESSAGE_BEGIN( MSG_ONE, gmsgShake, NULL, pPlayer);      // use the magic #1 for "one client"
            WRITE_SHORT( shake.amplitude );            // shake amount
            WRITE_SHORT( shake.duration );            // shake lasts this long
            WRITE_SHORT( shake.frequency );            // shake noise frequency
         MESSAGE_END();
      }
   }
}

unsigned short FixedUnsigned16( float value, float scale )
{
   int output;

   output = value * scale;
   if ( output < 0 )
      output = 0;
   if ( output > 0xFFFF )
      output = 0xFFFF;

   return (unsigned short)output;
}
